-------------------------------------------------------------------------------
--
-- Copyright (C) 2015
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-------------------------------------------------------------------------------

LIBRARY IEEE, common_lib, unb2b_board_lib, mm_lib;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.NUMERIC_STD.ALL;
USE common_lib.common_pkg.ALL;
USE common_lib.common_mem_pkg.ALL;
USE unb2b_board_lib.unb2b_board_pkg.ALL;
USE unb2b_board_lib.unb2b_board_peripherals_pkg.ALL;
USE mm_lib.mm_file_pkg.ALL;
USE mm_lib.mm_file_unb_pkg.ALL;
USE work.qsys_lofar2_unb2b_adc_pkg.ALL;


ENTITY mmm_lofar2_unb2b_adc IS
  GENERIC (
    g_sim         : BOOLEAN := FALSE; --FALSE: use QSYS; TRUE: use mm_file I/O
    g_sim_unb_nr  : NATURAL := 0;
    g_sim_node_nr : NATURAL := 0
  );
  PORT (
    mm_rst                   : IN  STD_LOGIC;
    mm_clk                   : IN  STD_LOGIC;

    pout_wdi                 : OUT STD_LOGIC;
                             
    -- Manual WDI override
    reg_wdi_mosi             : OUT t_mem_mosi;
    reg_wdi_miso             : IN  t_mem_miso;
                             
    -- system_info
    reg_unb_system_info_mosi : OUT t_mem_mosi;
    reg_unb_system_info_miso : IN  t_mem_miso;
    rom_unb_system_info_mosi : OUT t_mem_mosi;
    rom_unb_system_info_miso : IN  t_mem_miso;
                             
    -- UniBoard I2C sensors
    reg_unb_sens_mosi        : OUT t_mem_mosi; 
    reg_unb_sens_miso        : IN  t_mem_miso; 
                             
    reg_fpga_temp_sens_mosi   : OUT t_mem_mosi;
    reg_fpga_temp_sens_miso   : IN  t_mem_miso;
    reg_fpga_voltage_sens_mosi: OUT t_mem_mosi;
    reg_fpga_voltage_sens_miso: IN  t_mem_miso;

    reg_unb_pmbus_mosi       : OUT t_mem_mosi;
    reg_unb_pmbus_miso       : IN  t_mem_miso;

    -- PPSH
    reg_ppsh_mosi            : OUT t_mem_mosi; 
    reg_ppsh_miso            : IN  t_mem_miso; 
                             
    -- eth1g
    eth1g_mm_rst             : OUT STD_LOGIC;
    eth1g_tse_mosi           : OUT t_mem_mosi;  
    eth1g_tse_miso           : IN  t_mem_miso;  
    eth1g_reg_mosi           : OUT t_mem_mosi;  
    eth1g_reg_miso           : IN  t_mem_miso;  
    eth1g_reg_interrupt      : IN  STD_LOGIC; 
    eth1g_ram_mosi           : OUT t_mem_mosi;  
    eth1g_ram_miso           : IN  t_mem_miso;

    -- EPCS read
    reg_dpmm_data_mosi       : OUT t_mem_mosi;
    reg_dpmm_data_miso       : IN  t_mem_miso;
    reg_dpmm_ctrl_mosi       : OUT t_mem_mosi;
    reg_dpmm_ctrl_miso       : IN  t_mem_miso;

    -- EPCS write
    reg_mmdp_data_mosi       : OUT t_mem_mosi;
    reg_mmdp_data_miso       : IN  t_mem_miso;
    reg_mmdp_ctrl_mosi       : OUT t_mem_mosi;
    reg_mmdp_ctrl_miso       : IN  t_mem_miso;

    -- EPCS status/control
    reg_epcs_mosi            : OUT t_mem_mosi;
    reg_epcs_miso            : IN  t_mem_miso;

    -- Remote Update
    reg_remu_mosi            : OUT t_mem_mosi;
    reg_remu_miso            : IN  t_mem_miso;

    -- Jesd control
    jesd204b_mosi            : OUT t_mem_mosi;
    jesd204b_miso            : IN  t_mem_miso;

    -- BSN Monitor
    reg_bsn_monitor_input_mosi : OUT t_mem_mosi := c_mem_mosi_rst;
    reg_bsn_monitor_input_miso : IN  t_mem_miso := c_mem_miso_rst;

    -- MM wideband waveform generator registers [0,1,2,3] for signal paths [A,B,C,D]
    reg_wg_mosi_arr               : OUT t_mem_mosi_arr(11 DOWNTO 0) := (OTHERS=>c_mem_mosi_rst);  
    reg_wg_miso_arr               : IN  t_mem_miso_arr(11 DOWNTO 0);
    ram_wg_mosi_arr               : OUT t_mem_mosi_arr(11 DOWNTO 0) := (OTHERS=>c_mem_mosi_rst);  
    ram_wg_miso_arr               : IN  t_mem_miso_arr(11 DOWNTO 0);
    
    -- JESD databuffer
    ram_diag_data_buf_jesd_mosi   : OUT t_mem_mosi;
    ram_diag_data_buf_jesd_miso   : IN  t_mem_miso;
    reg_diag_data_buf_jesd_mosi   : OUT t_mem_mosi;
    reg_diag_data_buf_jesd_miso   : IN  t_mem_miso
  );
END mmm_lofar2_unb2b_adc;

ARCHITECTURE str OF mmm_lofar2_unb2b_adc IS

  CONSTANT c_sim_node_nr   : NATURAL := g_sim_node_nr;
  CONSTANT c_sim_node_type : STRING(1 TO 2):= "FN";

  SIGNAL i_reset_n         : STD_LOGIC;

BEGIN

  ----------------------------------------------------------------------------
  -- MM <-> file I/O for simulation. The files are created in $UPE/sim.
  ----------------------------------------------------------------------------
  gen_mm_file_io : IF g_sim = TRUE GENERATE

    u_mm_file_reg_unb_system_info : mm_file GENERIC MAP(mmf_unb_file_prefix(g_sim_unb_nr, c_sim_node_nr, c_sim_node_type) & "PIO_SYSTEM_INFO")
                                               PORT MAP(mm_rst, mm_clk, reg_unb_system_info_mosi, reg_unb_system_info_miso );

    u_mm_file_rom_unb_system_info : mm_file GENERIC MAP(mmf_unb_file_prefix(g_sim_unb_nr, c_sim_node_nr, c_sim_node_type) & "ROM_SYSTEM_INFO")
                                               PORT MAP(mm_rst, mm_clk, rom_unb_system_info_mosi, rom_unb_system_info_miso );

    u_mm_file_reg_wdi             : mm_file GENERIC MAP(mmf_unb_file_prefix(g_sim_unb_nr, c_sim_node_nr, c_sim_node_type) & "REG_WDI")
                                               PORT MAP(mm_rst, mm_clk, reg_wdi_mosi, reg_wdi_miso );

    u_mm_file_reg_unb_sens        : mm_file GENERIC MAP(mmf_unb_file_prefix(g_sim_unb_nr, c_sim_node_nr, c_sim_node_type) & "REG_UNB_SENS")
                                               PORT MAP(mm_rst, mm_clk, reg_unb_sens_mosi, reg_unb_sens_miso );

    u_mm_file_reg_unb_pmbus       : mm_file GENERIC MAP(mmf_unb_file_prefix(g_sim_unb_nr, c_sim_node_nr, c_sim_node_type) & "REG_UNB_PMBUS")
                                               PORT MAP(mm_rst, mm_clk, reg_unb_pmbus_mosi, reg_unb_pmbus_miso );

    u_mm_file_reg_fpga_temp_sens  : mm_file GENERIC MAP(mmf_unb_file_prefix(g_sim_unb_nr, c_sim_node_nr, c_sim_node_type) & "REG_FPGA_TEMP_SENS")
                                               PORT MAP(mm_rst, mm_clk, reg_fpga_temp_sens_mosi, reg_fpga_temp_sens_miso );

    u_mm_file_reg_fpga_voltage_sens :  mm_file GENERIC MAP(mmf_unb_file_prefix(g_sim_unb_nr, c_sim_node_nr, c_sim_node_type) & "REG_FPGA_VOLTAGE_SENS")
                                               PORT MAP(mm_rst, mm_clk, reg_fpga_voltage_sens_mosi, reg_fpga_voltage_sens_miso );

    u_mm_file_reg_ppsh            : mm_file GENERIC MAP(mmf_unb_file_prefix(g_sim_unb_nr, c_sim_node_nr, c_sim_node_type) & "PIO_PPS")
                                               PORT MAP(mm_rst, mm_clk, reg_ppsh_mosi, reg_ppsh_miso );

    -- Note: the eth1g RAM and TSE buses are only required by unb_osy on the NIOS as they provide the ethernet<->MM gateway.
    u_mm_file_reg_eth             : mm_file GENERIC MAP(mmf_unb_file_prefix(g_sim_unb_nr, c_sim_node_nr, c_sim_node_type) & "AVS_ETH_0_MMS_REG")
                                               PORT MAP(mm_rst, mm_clk, eth1g_reg_mosi, eth1g_reg_miso );

    ----------------------------------------------------------------------------
    -- Procedure that polls a sim control file that can be used to e.g. get
    -- the simulation time in ns
    ----------------------------------------------------------------------------
    mmf_poll_sim_ctrl_file(mm_clk, c_mmf_unb_file_path & "sim.ctrl", c_mmf_unb_file_path & "sim.stat");

  END GENERATE;

  i_reset_n <= NOT mm_rst;
  ----------------------------------------------------------------------------
  -- QSYS for synthesis
  ----------------------------------------------------------------------------
  gen_qsys : IF g_sim = FALSE GENERATE
    u_qsys : qsys_lofar2_unb2b_adc
    PORT MAP (

      clk_clk                                   => mm_clk,
      reset_reset_n                             => i_reset_n,

      -- the_pio_wdi: toggled by NIOS II application unb_osy. Connects to WDI via ctrl_unb2b_board.
      pio_wdi_external_connection_export        => pout_wdi,

      avs_eth_0_reset_export                    => eth1g_mm_rst,
      avs_eth_0_clk_export                      => OPEN,
      avs_eth_0_tse_address_export              => eth1g_tse_mosi.address(c_unb2b_board_peripherals_mm_reg_default.reg_tse_adr_w-1 DOWNTO 0),
      avs_eth_0_tse_write_export                => eth1g_tse_mosi.wr,
      avs_eth_0_tse_read_export                 => eth1g_tse_mosi.rd,
      avs_eth_0_tse_writedata_export            => eth1g_tse_mosi.wrdata(c_word_w-1 DOWNTO 0),
      avs_eth_0_tse_readdata_export             => eth1g_tse_miso.rddata(c_word_w-1 DOWNTO 0),
      avs_eth_0_tse_waitrequest_export          => eth1g_tse_miso.waitrequest,
      avs_eth_0_reg_address_export              => eth1g_reg_mosi.address(c_unb2b_board_peripherals_mm_reg_default.reg_eth_adr_w-1 DOWNTO 0),
      avs_eth_0_reg_write_export                => eth1g_reg_mosi.wr,
      avs_eth_0_reg_read_export                 => eth1g_reg_mosi.rd,
      avs_eth_0_reg_writedata_export            => eth1g_reg_mosi.wrdata(c_word_w-1 DOWNTO 0),
      avs_eth_0_reg_readdata_export             => eth1g_reg_miso.rddata(c_word_w-1 DOWNTO 0),
      avs_eth_0_ram_address_export              => eth1g_ram_mosi.address(c_unb2b_board_peripherals_mm_reg_default.ram_eth_adr_w-1 DOWNTO 0),
      avs_eth_0_ram_write_export                => eth1g_ram_mosi.wr,
      avs_eth_0_ram_read_export                 => eth1g_ram_mosi.rd,
      avs_eth_0_ram_writedata_export            => eth1g_ram_mosi.wrdata(c_word_w-1 DOWNTO 0),
      avs_eth_0_ram_readdata_export             => eth1g_ram_miso.rddata(c_word_w-1 DOWNTO 0),
      avs_eth_0_irq_export                      => eth1g_reg_interrupt,

      reg_unb_sens_reset_export                 => OPEN,
      reg_unb_sens_clk_export                   => OPEN,
      reg_unb_sens_address_export               => reg_unb_sens_mosi.address(c_unb2b_board_peripherals_mm_reg_default.reg_unb_sens_adr_w-1 DOWNTO 0),
      reg_unb_sens_write_export                 => reg_unb_sens_mosi.wr,
      reg_unb_sens_writedata_export             => reg_unb_sens_mosi.wrdata(c_word_w-1 DOWNTO 0),
      reg_unb_sens_read_export                  => reg_unb_sens_mosi.rd,
      reg_unb_sens_readdata_export              => reg_unb_sens_miso.rddata(c_word_w-1 DOWNTO 0),

      reg_unb_pmbus_reset_export                => OPEN,
      reg_unb_pmbus_clk_export                  => OPEN,
      reg_unb_pmbus_address_export              => reg_unb_pmbus_mosi.address(c_unb2b_board_peripherals_mm_reg_default.reg_unb_pmbus_adr_w-1 DOWNTO 0),
      reg_unb_pmbus_write_export                => reg_unb_pmbus_mosi.wr,
      reg_unb_pmbus_writedata_export            => reg_unb_pmbus_mosi.wrdata(c_word_w-1 DOWNTO 0),
      reg_unb_pmbus_read_export                 => reg_unb_pmbus_mosi.rd,
      reg_unb_pmbus_readdata_export             => reg_unb_pmbus_miso.rddata(c_word_w-1 DOWNTO 0),

      reg_fpga_temp_sens_reset_export           => OPEN,
      reg_fpga_temp_sens_clk_export             => OPEN,
      reg_fpga_temp_sens_address_export         => reg_fpga_temp_sens_mosi.address(c_unb2b_board_peripherals_mm_reg_default.reg_fpga_temp_sens_adr_w-1 DOWNTO 0),
      reg_fpga_temp_sens_write_export           => reg_fpga_temp_sens_mosi.wr,
      reg_fpga_temp_sens_writedata_export       => reg_fpga_temp_sens_mosi.wrdata(c_word_w-1 DOWNTO 0),
      reg_fpga_temp_sens_read_export            => reg_fpga_temp_sens_mosi.rd,
      reg_fpga_temp_sens_readdata_export        => reg_fpga_temp_sens_miso.rddata(c_word_w-1 DOWNTO 0),

      reg_fpga_voltage_sens_reset_export        => OPEN,
      reg_fpga_voltage_sens_clk_export          => OPEN,
      reg_fpga_voltage_sens_address_export      => reg_fpga_voltage_sens_mosi.address(c_unb2b_board_peripherals_mm_reg_default.reg_fpga_voltage_sens_adr_w-1 DOWNTO 0),
      reg_fpga_voltage_sens_write_export        => reg_fpga_voltage_sens_mosi.wr,
      reg_fpga_voltage_sens_writedata_export    => reg_fpga_voltage_sens_mosi.wrdata(c_word_w-1 DOWNTO 0),
      reg_fpga_voltage_sens_read_export         => reg_fpga_voltage_sens_mosi.rd,
      reg_fpga_voltage_sens_readdata_export     => reg_fpga_voltage_sens_miso.rddata(c_word_w-1 DOWNTO 0),

      rom_system_info_reset_export              => OPEN,
      rom_system_info_clk_export                => OPEN,
      rom_system_info_address_export            => rom_unb_system_info_mosi.address(c_unb2b_board_peripherals_mm_reg_default.rom_unb_system_info_adr_w-1 DOWNTO 0), 
      rom_system_info_write_export              => rom_unb_system_info_mosi.wr,
      rom_system_info_writedata_export          => rom_unb_system_info_mosi.wrdata(c_word_w-1 DOWNTO 0),
      rom_system_info_read_export               => rom_unb_system_info_mosi.rd,
      rom_system_info_readdata_export           => rom_unb_system_info_miso.rddata(c_word_w-1 DOWNTO 0),

      pio_system_info_reset_export              => OPEN,
      pio_system_info_clk_export                => OPEN,
      pio_system_info_address_export            => reg_unb_system_info_mosi.address(c_unb2b_board_peripherals_mm_reg_default.reg_unb_system_info_adr_w-1 DOWNTO 0), 
      pio_system_info_write_export              => reg_unb_system_info_mosi.wr,
      pio_system_info_writedata_export          => reg_unb_system_info_mosi.wrdata(c_word_w-1 DOWNTO 0),
      pio_system_info_read_export               => reg_unb_system_info_mosi.rd,
      pio_system_info_readdata_export           => reg_unb_system_info_miso.rddata(c_word_w-1 DOWNTO 0),

      pio_pps_reset_export                      => OPEN,
      pio_pps_clk_export                        => OPEN,
      pio_pps_address_export                    => reg_ppsh_mosi.address(c_unb2b_board_peripherals_mm_reg_default.reg_ppsh_adr_w-1 DOWNTO 0),
      pio_pps_write_export                      => reg_ppsh_mosi.wr,
      pio_pps_writedata_export                  => reg_ppsh_mosi.wrdata(c_word_w-1 DOWNTO 0),
      pio_pps_read_export                       => reg_ppsh_mosi.rd,
      pio_pps_readdata_export                   => reg_ppsh_miso.rddata(c_word_w-1 DOWNTO 0),

      reg_wdi_reset_export                      => OPEN,
      reg_wdi_clk_export                        => OPEN,
      reg_wdi_address_export                    => reg_wdi_mosi.address(0 DOWNTO 0),
      reg_wdi_write_export                      => reg_wdi_mosi.wr,
      reg_wdi_writedata_export                  => reg_wdi_mosi.wrdata(c_word_w-1 DOWNTO 0),
      reg_wdi_read_export                       => reg_wdi_mosi.rd,
      reg_wdi_readdata_export                   => reg_wdi_miso.rddata(c_word_w-1 DOWNTO 0),

      reg_remu_reset_export                     => OPEN,
      reg_remu_clk_export                       => OPEN,
      reg_remu_address_export                   => reg_remu_mosi.address(c_unb2b_board_peripherals_mm_reg_default.reg_remu_adr_w-1 DOWNTO 0),
      reg_remu_write_export                     => reg_remu_mosi.wr,
      reg_remu_writedata_export                 => reg_remu_mosi.wrdata(c_word_w-1 DOWNTO 0),
      reg_remu_read_export                      => reg_remu_mosi.rd,
      reg_remu_readdata_export                  => reg_remu_miso.rddata(c_word_w-1 DOWNTO 0),

      jesd204b_reset_export                     => OPEN,
      jesd204b_clk_export                       => OPEN,
      jesd204b_address_export                   => jesd204b_mosi.address(11 DOWNTO 0),
      jesd204b_write_export                     => jesd204b_mosi.wr,
      jesd204b_writedata_export                 => jesd204b_mosi.wrdata(c_word_w-1 DOWNTO 0),
      jesd204b_read_export                      => jesd204b_mosi.rd,
      jesd204b_readdata_export                  => jesd204b_miso.rddata(c_word_w-1 DOWNTO 0),

      reg_bsn_monitor_input_address_export      => reg_bsn_monitor_input_mosi.address(7 DOWNTO 0),
      reg_bsn_monitor_input_clk_export          => OPEN,
      reg_bsn_monitor_input_read_export         => reg_bsn_monitor_input_mosi.rd,
      reg_bsn_monitor_input_readdata_export     => reg_bsn_monitor_input_miso.rddata(c_word_w-1 DOWNTO 0),
      reg_bsn_monitor_input_reset_export        => OPEN,
      reg_bsn_monitor_input_write_export        => reg_bsn_monitor_input_mosi.wr,
      reg_bsn_monitor_input_writedata_export    => reg_bsn_monitor_input_mosi.wrdata(c_word_w-1 DOWNTO 0),

      -- waveform generators, just use 4 of 12 instances for now
      reg_wg_0_clk_export                       => OPEN,
      reg_wg_0_reset_export                     => OPEN,
      reg_wg_0_address_export                   => reg_wg_mosi_arr(0).address(1 DOWNTO 0),
      reg_wg_0_read_export                      => reg_wg_mosi_arr(0).rd,
      reg_wg_0_readdata_export                  => reg_wg_miso_arr(0).rddata(c_word_w-1 DOWNTO 0),
      reg_wg_0_write_export                     => reg_wg_mosi_arr(0).wr,
      reg_wg_0_writedata_export                 => reg_wg_mosi_arr(0).wrdata(c_word_w-1 DOWNTO 0),

      reg_wg_1_clk_export                       => OPEN,
      reg_wg_1_reset_export                     => OPEN,
      reg_wg_1_address_export                   => reg_wg_mosi_arr(1).address(1 DOWNTO 0),
      reg_wg_1_read_export                      => reg_wg_mosi_arr(1).rd,
      reg_wg_1_readdata_export                  => reg_wg_miso_arr(1).rddata(c_word_w-1 DOWNTO 0),
      reg_wg_1_write_export                     => reg_wg_mosi_arr(1).wr,
      reg_wg_1_writedata_export                 => reg_wg_mosi_arr(1).wrdata(c_word_w-1 DOWNTO 0),

      reg_wg_2_clk_export                       => OPEN,
      reg_wg_2_reset_export                     => OPEN,
      reg_wg_2_address_export                   => reg_wg_mosi_arr(2).address(1 DOWNTO 0),
      reg_wg_2_read_export                      => reg_wg_mosi_arr(2).rd,
      reg_wg_2_readdata_export                  => reg_wg_miso_arr(2).rddata(c_word_w-1 DOWNTO 0),
      reg_wg_2_write_export                     => reg_wg_mosi_arr(2).wr,
      reg_wg_2_writedata_export                 => reg_wg_mosi_arr(2).wrdata(c_word_w-1 DOWNTO 0),

      reg_wg_3_clk_export                       => OPEN,
      reg_wg_3_reset_export                     => OPEN,
      reg_wg_3_address_export                   => reg_wg_mosi_arr(3).address(1 DOWNTO 0),
      reg_wg_3_read_export                      => reg_wg_mosi_arr(3).rd,
      reg_wg_3_readdata_export                  => reg_wg_miso_arr(3).rddata(c_word_w-1 DOWNTO 0),
      reg_wg_3_write_export                     => reg_wg_mosi_arr(3).wr,
      reg_wg_3_writedata_export                 => reg_wg_mosi_arr(3).wrdata(c_word_w-1 DOWNTO 0),

      --reg_wg_4_clk_export                       => OPEN,
      --reg_wg_4_reset_export                     => OPEN,
      --reg_wg_4_address_export                   => reg_wg_mosi_arr(4).address(1 DOWNTO 0),
      --reg_wg_4_read_export                      => reg_wg_mosi_arr(4).rd,
      --reg_wg_4_readdata_export                  => reg_wg_miso_arr(4).rddata(c_word_w-1 DOWNTO 0),
      --reg_wg_4_write_export                     => reg_wg_mosi_arr(4).wr,
      --reg_wg_4_writedata_export                 => reg_wg_mosi_arr(4).wrdata(c_word_w-1 DOWNTO 0),

      --reg_wg_5_clk_export                       => OPEN,
      --reg_wg_5_reset_export                     => OPEN,
      --reg_wg_5_address_export                   => reg_wg_mosi_arr(5).address(1 DOWNTO 0),
      --reg_wg_5_read_export                      => reg_wg_mosi_arr(5).rd,
      --reg_wg_5_readdata_export                  => reg_wg_miso_arr(5).rddata(c_word_w-1 DOWNTO 0),
      --reg_wg_5_write_export                     => reg_wg_mosi_arr(5).wr,
      --reg_wg_5_writedata_export                 => reg_wg_mosi_arr(5).wrdata(c_word_w-1 DOWNTO 0),

      --reg_wg_6_clk_export                       => OPEN,
      --reg_wg_6_reset_export                     => OPEN,
      --reg_wg_6_address_export                   => reg_wg_mosi_arr(6).address(1 DOWNTO 0),
      --reg_wg_6_read_export                      => reg_wg_mosi_arr(6).rd,
      --reg_wg_6_readdata_export                  => reg_wg_miso_arr(6).rddata(c_word_w-1 DOWNTO 0),
      --reg_wg_6_write_export                     => reg_wg_mosi_arr(6).wr,
      --reg_wg_6_writedata_export                 => reg_wg_mosi_arr(6).wrdata(c_word_w-1 DOWNTO 0),

      --reg_wg_7_clk_export                       => OPEN,
      --reg_wg_7_reset_export                     => OPEN,
      --reg_wg_7_address_export                   => reg_wg_mosi_arr(7).address(1 DOWNTO 0),
      --reg_wg_7_read_export                      => reg_wg_mosi_arr(7).rd,
      --reg_wg_7_readdata_export                  => reg_wg_miso_arr(7).rddata(c_word_w-1 DOWNTO 0),
      --reg_wg_7_write_export                     => reg_wg_mosi_arr(7).wr,
      --reg_wg_7_writedata_export                 => reg_wg_mosi_arr(7).wrdata(c_word_w-1 DOWNTO 0),

      --reg_wg_8_clk_export                       => OPEN,
      --reg_wg_8_reset_export                     => OPEN,
      --reg_wg_8_address_export                   => reg_wg_mosi_arr(8).address(1 DOWNTO 0),
      --reg_wg_8_read_export                      => reg_wg_mosi_arr(8).rd,
      --reg_wg_8_readdata_export                  => reg_wg_miso_arr(8).rddata(c_word_w-1 DOWNTO 0),
      --reg_wg_8_write_export                     => reg_wg_mosi_arr(8).wr,
      --reg_wg_8_writedata_export                 => reg_wg_mosi_arr(8).wrdata(c_word_w-1 DOWNTO 0),

      --reg_wg_9_clk_export                       => OPEN,
      --reg_wg_9_reset_export                     => OPEN,
      --reg_wg_9_address_export                   => reg_wg_mosi_arr(9).address(1 DOWNTO 0),
      --reg_wg_9_read_export                      => reg_wg_mosi_arr(9).rd,
      --reg_wg_9_readdata_export                  => reg_wg_miso_arr(9).rddata(c_word_w-1 DOWNTO 0),
      --reg_wg_9_write_export                     => reg_wg_mosi_arr(9).wr,
      --reg_wg_9_writedata_export                 => reg_wg_mosi_arr(9).wrdata(c_word_w-1 DOWNTO 0),

      --reg_wg_10_clk_export                       => OPEN,
      --reg_wg_10_reset_export                     => OPEN,
      --reg_wg_10_address_export                   => reg_wg_mosi_arr(10).address(1 DOWNTO 0),
      --reg_wg_10_read_export                      => reg_wg_mosi_arr(10).rd,
      --reg_wg_10_readdata_export                  => reg_wg_miso_arr(10).rddata(c_word_w-1 DOWNTO 0),
      --reg_wg_10_write_export                     => reg_wg_mosi_arr(10).wr,
      --reg_wg_10_writedata_export                 => reg_wg_mosi_arr(10).wrdata(c_word_w-1 DOWNTO 0),

      --reg_wg_11_clk_export                       => OPEN,
      --reg_wg_11_reset_export                     => OPEN,
      --reg_wg_11_address_export                   => reg_wg_mosi_arr(11).address(1 DOWNTO 0),
      --reg_wg_11_read_export                      => reg_wg_mosi_arr(11).rd,
      --reg_wg_11_readdata_export                  => reg_wg_miso_arr(11).rddata(c_word_w-1 DOWNTO 0),
      --reg_wg_11_write_export                     => reg_wg_mosi_arr(11).wr,
      --reg_wg_11_writedata_export                 => reg_wg_mosi_arr(11).wrdata(c_word_w-1 DOWNTO 0),

      ram_wg_0_clk_export                       => OPEN,
      ram_wg_0_reset_export                     => OPEN,
      ram_wg_0_address_export                   => ram_wg_mosi_arr(0).address(9 DOWNTO 0),
      ram_wg_0_read_export                      => ram_wg_mosi_arr(0).rd,
      ram_wg_0_readdata_export                  => ram_wg_miso_arr(0).rddata(c_word_w-1 DOWNTO 0),
      ram_wg_0_write_export                     => ram_wg_mosi_arr(0).wr,
      ram_wg_0_writedata_export                 => ram_wg_mosi_arr(0).wrdata(c_word_w-1 DOWNTO 0),

      ram_wg_1_clk_export                       => OPEN,
      ram_wg_1_reset_export                     => OPEN,
      ram_wg_1_address_export                   => ram_wg_mosi_arr(1).address(9 DOWNTO 0),
      ram_wg_1_read_export                      => ram_wg_mosi_arr(1).rd,
      ram_wg_1_readdata_export                  => ram_wg_miso_arr(1).rddata(c_word_w-1 DOWNTO 0),
      ram_wg_1_write_export                     => ram_wg_mosi_arr(1).wr,
      ram_wg_1_writedata_export                 => ram_wg_mosi_arr(1).wrdata(c_word_w-1 DOWNTO 0),

      ram_wg_2_clk_export                       => OPEN,
      ram_wg_2_reset_export                     => OPEN,
      ram_wg_2_address_export                   => ram_wg_mosi_arr(2).address(9 DOWNTO 0),
      ram_wg_2_read_export                      => ram_wg_mosi_arr(2).rd,
      ram_wg_2_readdata_export                  => ram_wg_miso_arr(2).rddata(c_word_w-1 DOWNTO 0),
      ram_wg_2_write_export                     => ram_wg_mosi_arr(2).wr,
      ram_wg_2_writedata_export                 => ram_wg_mosi_arr(2).wrdata(c_word_w-1 DOWNTO 0),

      ram_wg_3_clk_export                       => OPEN,
      ram_wg_3_reset_export                     => OPEN,
      ram_wg_3_address_export                   => ram_wg_mosi_arr(3).address(9 DOWNTO 0),
      ram_wg_3_read_export                      => ram_wg_mosi_arr(3).rd,
      ram_wg_3_readdata_export                  => ram_wg_miso_arr(3).rddata(c_word_w-1 DOWNTO 0),
      ram_wg_3_write_export                     => ram_wg_mosi_arr(3).wr,
      ram_wg_3_writedata_export                 => ram_wg_mosi_arr(3).wrdata(c_word_w-1 DOWNTO 0),

      --ram_wg_4_clk_export                       => OPEN,
      --ram_wg_4_reset_export                     => OPEN,
      --ram_wg_4_address_export                   => ram_wg_mosi_arr(4).address(9 DOWNTO 0),
      --ram_wg_4_read_export                      => ram_wg_mosi_arr(4).rd,
      --ram_wg_4_readdata_export                  => ram_wg_miso_arr(4).rddata(c_word_w-1 DOWNTO 0),
      --ram_wg_4_write_export                     => ram_wg_mosi_arr(4).wr,
      --ram_wg_4_writedata_export                 => ram_wg_mosi_arr(4).wrdata(c_word_w-1 DOWNTO 0),

      --ram_wg_5_clk_export                       => OPEN,
      --ram_wg_5_reset_export                     => OPEN,
      --ram_wg_5_address_export                   => ram_wg_mosi_arr(5).address(9 DOWNTO 0),
      --ram_wg_5_read_export                      => ram_wg_mosi_arr(5).rd,
      --ram_wg_5_readdata_export                  => ram_wg_miso_arr(5).rddata(c_word_w-1 DOWNTO 0),
      --ram_wg_5_write_export                     => ram_wg_mosi_arr(5).wr,
      --ram_wg_5_writedata_export                 => ram_wg_mosi_arr(5).wrdata(c_word_w-1 DOWNTO 0),

      --ram_wg_6_clk_export                       => OPEN,
      --ram_wg_6_reset_export                     => OPEN,
      --ram_wg_6_address_export                   => ram_wg_mosi_arr(6).address(9 DOWNTO 0),
      --ram_wg_6_read_export                      => ram_wg_mosi_arr(6).rd,
      --ram_wg_6_readdata_export                  => ram_wg_miso_arr(6).rddata(c_word_w-1 DOWNTO 0),
      --ram_wg_6_write_export                     => ram_wg_mosi_arr(6).wr,
      --ram_wg_6_writedata_export                 => ram_wg_mosi_arr(6).wrdata(c_word_w-1 DOWNTO 0),

      --ram_wg_7_clk_export                       => OPEN,
      --ram_wg_7_reset_export                     => OPEN,
      --ram_wg_7_address_export                   => ram_wg_mosi_arr(7).address(9 DOWNTO 0),
      --ram_wg_7_read_export                      => ram_wg_mosi_arr(7).rd,
      --ram_wg_7_readdata_export                  => ram_wg_miso_arr(7).rddata(c_word_w-1 DOWNTO 0),
      --ram_wg_7_write_export                     => ram_wg_mosi_arr(7).wr,
      --ram_wg_7_writedata_export                 => ram_wg_mosi_arr(7).wrdata(c_word_w-1 DOWNTO 0),

      --ram_wg_8_clk_export                       => OPEN,
      --ram_wg_8_reset_export                     => OPEN,
      --ram_wg_8_address_export                   => ram_wg_mosi_arr(8).address(9 DOWNTO 0),
      --ram_wg_8_read_export                      => ram_wg_mosi_arr(8).rd,
      --ram_wg_8_readdata_export                  => ram_wg_miso_arr(8).rddata(c_word_w-1 DOWNTO 0),
      --ram_wg_8_write_export                     => ram_wg_mosi_arr(8).wr,
      --ram_wg_8_writedata_export                 => ram_wg_mosi_arr(8).wrdata(c_word_w-1 DOWNTO 0),

      --ram_wg_9_clk_export                       => OPEN,
      --ram_wg_9_reset_export                     => OPEN,
      --ram_wg_9_address_export                   => ram_wg_mosi_arr(9).address(9 DOWNTO 0),
      --ram_wg_9_read_export                      => ram_wg_mosi_arr(9).rd,
      --ram_wg_9_readdata_export                  => ram_wg_miso_arr(9).rddata(c_word_w-1 DOWNTO 0),
      --ram_wg_9_write_export                     => ram_wg_mosi_arr(9).wr,
      --ram_wg_9_writedata_export                 => ram_wg_mosi_arr(9).wrdata(c_word_w-1 DOWNTO 0),

      --ram_wg_10_clk_export                       => OPEN,
      --ram_wg_10_reset_export                     => OPEN,
      --ram_wg_10_address_export                   => ram_wg_mosi_arr(10).address(9 DOWNTO 0),
      --ram_wg_10_read_export                      => ram_wg_mosi_arr(10).rd,
      --ram_wg_10_readdata_export                  => ram_wg_miso_arr(10).rddata(c_word_w-1 DOWNTO 0),
      --ram_wg_10_write_export                     => ram_wg_mosi_arr(10).wr,
      --ram_wg_10_writedata_export                 => ram_wg_mosi_arr(10).wrdata(c_word_w-1 DOWNTO 0),

      --ram_wg_11_clk_export                       => OPEN,
      --ram_wg_11_reset_export                     => OPEN,
      --ram_wg_11_address_export                   => ram_wg_mosi_arr(11).address(9 DOWNTO 0),
      --ram_wg_11_read_export                      => ram_wg_mosi_arr(11).rd,
      --ram_wg_11_readdata_export                  => ram_wg_miso_arr(11).rddata(c_word_w-1 DOWNTO 0),
      --ram_wg_11_write_export                     => ram_wg_mosi_arr(11).wr,
      --ram_wg_11_writedata_export                 => ram_wg_mosi_arr(11).wrdata(c_word_w-1 DOWNTO 0),

      reg_epcs_reset_export                     => OPEN,
      reg_epcs_clk_export                       => OPEN,
      reg_epcs_address_export                   => reg_epcs_mosi.address(c_unb2b_board_peripherals_mm_reg_default.reg_epcs_adr_w-1 DOWNTO 0),
      reg_epcs_write_export                     => reg_epcs_mosi.wr,
      reg_epcs_writedata_export                 => reg_epcs_mosi.wrdata(c_word_w-1 DOWNTO 0),
      reg_epcs_read_export                      => reg_epcs_mosi.rd,
      reg_epcs_readdata_export                  => reg_epcs_miso.rddata(c_word_w-1 DOWNTO 0),

      reg_dpmm_ctrl_reset_export                => OPEN,
      reg_dpmm_ctrl_clk_export                  => OPEN,
      reg_dpmm_ctrl_address_export              => reg_dpmm_ctrl_mosi.address(0 DOWNTO 0),
      reg_dpmm_ctrl_write_export                => reg_dpmm_ctrl_mosi.wr,
      reg_dpmm_ctrl_writedata_export            => reg_dpmm_ctrl_mosi.wrdata(c_word_w-1 DOWNTO 0),
      reg_dpmm_ctrl_read_export                 => reg_dpmm_ctrl_mosi.rd,
      reg_dpmm_ctrl_readdata_export             => reg_dpmm_ctrl_miso.rddata(c_word_w-1 DOWNTO 0),

      reg_mmdp_data_reset_export                => OPEN,
      reg_mmdp_data_clk_export                  => OPEN,
      reg_mmdp_data_address_export              => reg_mmdp_data_mosi.address(0 DOWNTO 0),
      reg_mmdp_data_write_export                => reg_mmdp_data_mosi.wr,
      reg_mmdp_data_writedata_export            => reg_mmdp_data_mosi.wrdata(c_word_w-1 DOWNTO 0),
      reg_mmdp_data_read_export                 => reg_mmdp_data_mosi.rd,
      reg_mmdp_data_readdata_export             => reg_mmdp_data_miso.rddata(c_word_w-1 DOWNTO 0),

      reg_dpmm_data_reset_export                => OPEN,
      reg_dpmm_data_clk_export                  => OPEN,
      reg_dpmm_data_address_export              => reg_dpmm_data_mosi.address(0 DOWNTO 0),
      reg_dpmm_data_read_export                 => reg_dpmm_data_mosi.rd,
      reg_dpmm_data_readdata_export             => reg_dpmm_data_miso.rddata(c_word_w-1 DOWNTO 0),
      reg_dpmm_data_write_export                => reg_dpmm_data_mosi.wr,
      reg_dpmm_data_writedata_export            => reg_dpmm_data_mosi.wrdata(c_word_w-1 DOWNTO 0),

      reg_mmdp_ctrl_reset_export                => OPEN,
      reg_mmdp_ctrl_clk_export                  => OPEN,
      reg_mmdp_ctrl_address_export              => reg_mmdp_ctrl_mosi.address(0 DOWNTO 0),
      reg_mmdp_ctrl_read_export                 => reg_mmdp_ctrl_mosi.rd,
      reg_mmdp_ctrl_readdata_export             => reg_mmdp_ctrl_miso.rddata(c_word_w-1 DOWNTO 0),
      reg_mmdp_ctrl_write_export                => reg_mmdp_ctrl_mosi.wr,
      reg_mmdp_ctrl_writedata_export            => reg_mmdp_ctrl_mosi.wrdata(c_word_w-1 DOWNTO 0),


      ram_diag_data_buf_jesd_clk_export         => OPEN,
      ram_diag_data_buf_jesd_reset_export       => OPEN,
      ram_diag_data_buf_jesd_address_export     => ram_diag_data_buf_jesd_mosi.address(17-1 DOWNTO 0),
      ram_diag_data_buf_jesd_write_export       => ram_diag_data_buf_jesd_mosi.wr,
      ram_diag_data_buf_jesd_writedata_export   => ram_diag_data_buf_jesd_mosi.wrdata(c_word_w-1 DOWNTO 0),
      ram_diag_data_buf_jesd_read_export        => ram_diag_data_buf_jesd_mosi.rd,
      ram_diag_data_buf_jesd_readdata_export    => ram_diag_data_buf_jesd_miso.rddata(c_word_w-1 DOWNTO 0),

      reg_diag_data_buf_jesd_reset_export       => OPEN,
      reg_diag_data_buf_jesd_clk_export         => OPEN,
      reg_diag_data_buf_jesd_address_export     => reg_diag_data_buf_jesd_mosi.address(12-1 DOWNTO 0),
      reg_diag_data_buf_jesd_write_export       => reg_diag_data_buf_jesd_mosi.wr,
      reg_diag_data_buf_jesd_writedata_export   => reg_diag_data_buf_jesd_mosi.wrdata(c_word_w-1 DOWNTO 0),
      reg_diag_data_buf_jesd_read_export        => reg_diag_data_buf_jesd_mosi.rd,
      reg_diag_data_buf_jesd_readdata_export    => reg_diag_data_buf_jesd_miso.rddata(c_word_w-1 DOWNTO 0)

    );
  END GENERATE;
END str;
